C     WOLFRAM RUEHAAK, 06/07/15
C     HARMONIC MEAN OF HYCON (HYDRAULIC CONDUCTIVITY)
C
      FUNCTION HMHYCON(I,J,K,IP2,JP2,KP2,DIR)
      
      USE FELDER
      USE NO

      IMPLICIT NONE

      include 'genrl.inc'
      include 'genrlc.inc'
      include 'temp.inc'
      include 'nofcoef.fi'
      include 'nogrid.fi'
      
      INTEGER           IP2,JP2,KP2,DIR
      DOUBLE PRECISION  HMHYCON,ANI,FACA,D1,D2,D3
      

C ********************** DEFINITION ************************************
C           E,N,T,W,S,B
C           1,2,3,4,5,6
C **********************************************************************
!      
! Die Diffusionskoeffizienten, z.B. a_e, lassen sich mittels
! harmonischer Mittlung \citep{Patankar} interpolieren:\\
! a_e = deltax_e1 * (2 * a_P * a_E)/(deltax_P1 * a_E + deltax_E1 * a_P)
! Hier sind:
! - deltax_e1 die euklidische Distanz von P nach E 
!                  also von der Mitte von P (X,Y,Z) bis zur Mitte von E (X+1,Y,Z)
! - deltax_P1 die euklidische Distanz von w nach e in dem ELEMENT P (X,Y,Z)
! - deltax_E1 die euklidische Distanz von w nach e in dem ELEMENT E (X+1,Y,Z)
!

C EAST /WEST
      IF(DIR.EQ.1.OR.DIR.EQ.4)THEN
      dxx2 = CALCED(
     +SMALLE(I,J,K,1),SMALLE(I,J,K,2),SMALLE(I,J,K,3),
     +SMALLW(I,J,K,1),SMALLW(I,J,K,2),SMALLW(I,J,K,3))
      dxx3 = CALCED(
     +SMALLE(IP2,JP2,KP2,1),SMALLE(IP2,JP2,KP2,2),SMALLE(IP2,JP2,KP2,3),
     +SMALLW(IP2,JP2,KP2,1),SMALLW(IP2,JP2,KP2,2),SMALLW(IP2,JP2,KP2,3))
      
C      ANI = ANISOI(I,J,K)
      
      D1 = ABS(CENTER(I,J,K,1) - CENTER(IP2,JP2,KP2,1))
      D2 = ABS(CENTER(I,J,K,2) - CENTER(IP2,JP2,KP2,2))
      D3 = ABS(CENTER(I,J,K,3) - CENTER(IP2,JP2,KP2,3))      
      ANI = ((ANISOI(I,J,K)*D1)+(ANISOJ(I,J,K)*D2)+(1.D0*D3))/(D1+D2+D3)
      
C NORTH / SOUTH
      ELSEIF(DIR.EQ.2.OR.DIR.EQ.5)THEN
      dxx2 = CALCED(
     +SMALLN(I,J,K,1),SMALLN(I,J,K,2),SMALLN(I,J,K,3),
     +SMALLS(I,J,K,1),SMALLS(I,J,K,2),SMALLS(I,J,K,3))
      dxx3 = CALCED(
     +SMALLN(IP2,JP2,KP2,1),SMALLN(IP2,JP2,KP2,2),SMALLN(IP2,JP2,KP2,3),
     +SMALLS(IP2,JP2,KP2,1),SMALLS(IP2,JP2,KP2,2),SMALLS(IP2,JP2,KP2,3))

C      ANI = ANISOJ(I,J,K)
      D1 = ABS(CENTER(I,J,K,1) - CENTER(IP2,JP2,KP2,1))
      D2 = ABS(CENTER(I,J,K,2) - CENTER(IP2,JP2,KP2,2))
      D3 = ABS(CENTER(I,J,K,3) - CENTER(IP2,JP2,KP2,3))
      ANI = ((ANISOI(I,J,K)*D1)+(ANISOJ(I,J,K)*D2)+(1.D0*D3))/(D1+D2+D3)
      
C TOP / BOTTOM
      ELSEIF(DIR.EQ.3.OR.DIR.EQ.6)THEN
      dxx2 = CALCED(
     +SMALLT(I,J,K,1),SMALLT(I,J,K,2),SMALLT(I,J,K,3),
     +SMALLB(I,J,K,1),SMALLB(I,J,K,2),SMALLB(I,J,K,3))
      dxx3 = CALCED(
     +SMALLT(IP2,JP2,KP2,1),SMALLT(IP2,JP2,KP2,2),SMALLT(IP2,JP2,KP2,3),
     +SMALLB(IP2,JP2,KP2,1),SMALLB(IP2,JP2,KP2,2),SMALLB(IP2,JP2,KP2,3))

C      ANI = 1.D0
      D1 = ABS(CENTER(I,J,K,1) - CENTER(IP2,JP2,KP2,1))
      D2 = ABS(CENTER(I,J,K,2) - CENTER(IP2,JP2,KP2,2))
      D3 = ABS(CENTER(I,J,K,3) - CENTER(IP2,JP2,KP2,3))
      ANI = ((ANISOI(I,J,K)*D1)+(ANISOJ(I,J,K)*D2)+(1.D0*D3))/(D1+D2+D3)

      ENDIF
      
      dxx1 = CALCED(
     +CENTER(I,J,K,1),CENTER(I,J,K,2),CENTER(I,J,K,3),
     +CENTER(IP2,JP2,KP2,1),CENTER(IP2,JP2,KP2,2),CENTER(IP2,JP2,KP2,3))
     
      aa1 = DABS(HYCON(I,J,K))
      aa2 = DABS(HYCON(IP2,JP2,KP2))
      FACA = CALCFA(aa1,aa2,dxx1,dxx2,dxx3)

C      WRITE(*,*) ANISOI(I,J,K),ANISOJ(I,J,K)
C      WRITE(*,'(A25,7(I3,1X),F10.4)') 'I,J,K,IP2,JP2,KP2,DIR,ANI: ',
C     + I,J,K,IP2,JP2,KP2,DIR,ANI
     
      HMHYCON = FACA * ANI
      
      RETURN
      END
