C
C 
C @author Wolfram Ruehaak
C LGPL license
C generate 8 tetrahedrons from one hexahedron
C 
C The procedure is inspired by 
C Vinokur, M., 1989. 
C An analysis of finite-difference and finite-volume formulations of conservation laws,
C Journal of Comp. Physics, 81, 1-52.
C 
C A artifical center of the hexaeder is constructed first, arround this  center
C 12 tetraeder are constructed.
C For each of these tetraeders a volume and center of gravity can now be constructed  
C 
C
C *********************************************************************
C     Aufsicht (oben ist Flaeche 5, unten ist Flaeche 6)
C
C     Obere Ebene
C
C          |         A3          |
C         ---------------------------
C          |K4                   |K3
C          |   \                 |
C          |     \               |
C          |       \             |
C       A4 |         A5          | A2
C          |           \         |
C          |             \       |
C          |               \     |
C          |                 \   |
C          |                   \ |
C         -------------------------
C          |K1       A1          |K2
C
C
C     Untere Ebene
C
C          |         A3          |
C         --------------------------
C          |K8                   |K7
C          |                     |
C          |                     |
C          |                     |
C       A4 |         A6          | A2
C          |                     |
C          |                     |
C          |                     |
C          |                     |
C          |                     |
C         -------------------------
C          |K5       A1          |K6
C
C
C       'K*' steht fuer Knoten/Vertice/Knode
C       'A*' steht fuer Area/Flaeche
C
C     K* ist definiert ueber XWR0(I),YWR0(I),ZWR0(I)
C
C *********************************************************************
      SUBROUTINE HEX2TET(X,Y,Z,TETRAEDER)
C
C **********************************************************************
C *    The knode (or vertices) enumeration ic CCW from bottom to top with z increasing.
C *    At point (0,0,0) is the first knode, at (1,0,0) the second, ...
C *    at point (0,0,1) is the fifth knode, at (1,0,1) the sixth, ...
C *
C **********************************************************************
C
      IMPLICIT NONE

      DOUBLE PRECISION X(8), Y(8), Z(8)
      DOUBLE PRECISION CENTER(3)
      DOUBLE PRECISION TETRAEDER(3,12,4)
      DOUBLE PRECISION COORD(3,8)
      INTEGER C, V
            
      CENTER(1) = 0.125 * (X(1)+X(2)+X(3)+X(4)+X(5)+X(6)+X(7)+X(8))
      CENTER(2) = 0.125 * (Y(1)+Y(2)+Y(3)+Y(4)+Y(5)+Y(6)+Y(7)+Y(8))
      CENTER(3) = 0.125 * (Z(1)+Z(2)+Z(3)+Z(4)+Z(5)+Z(6)+Z(7)+Z(8))
          
            DO V = 1, 8
                    COORD(1,V) = X(V)
                    COORD(2,V) = Y(V)
                    COORD(3,V) = Z(V)
            ENDDO
          
          
            DO C = 1, 3
C OVER ALL 3 COORDINATE DIRECTIONS
C                   1.  TOP
                    TETRAEDER(C,1,1) = CENTER(C)
                    TETRAEDER(C,1,2) = COORD(C,1)
                    TETRAEDER(C,1,3) = COORD(C,2)
                    TETRAEDER(C,1,4) = COORD(C,4)
C                   2.  TOP
                    TETRAEDER(C,2,1) = CENTER(C)
                    TETRAEDER(C,2,2) = COORD(C,2)
                    TETRAEDER(C,2,3) = COORD(C,3)
                    TETRAEDER(C,2,4) = COORD(C,4)
C                   3.  WEST
                    TETRAEDER(C,3,1) = CENTER(C)
                    TETRAEDER(C,3,2) = COORD(C,1)
                    TETRAEDER(C,3,3) = COORD(C,5)
                    TETRAEDER(C,3,4) = COORD(C,8)
C                   4.  WEST
                    TETRAEDER(C,4,1) = CENTER(C)
                    TETRAEDER(C,4,2) = COORD(C,1)
                    TETRAEDER(C,4,3) = COORD(C,4)
                    TETRAEDER(C,4,4) = COORD(C,8)
C                   5.  EAST
                    TETRAEDER(C,5,1) = CENTER(C)
                    TETRAEDER(C,5,2) = COORD(C,2)
                    TETRAEDER(C,5,3) = COORD(C,6)
                    TETRAEDER(C,5,4) = COORD(C,7)
C                   6.  EAST
                    TETRAEDER(C,6,1) = CENTER(C)
                    TETRAEDER(C,6,2) = COORD(C,2)
                    TETRAEDER(C,6,3) = COORD(C,3)
                    TETRAEDER(C,6,4) = COORD(C,7)
C                   7.  NORTH
                    TETRAEDER(C,7,1) = CENTER(C)
                    TETRAEDER(C,7,2) = COORD(C,4)
                    TETRAEDER(C,7,3) = COORD(C,8)
                    TETRAEDER(C,7,4) = COORD(C,7)
C                   8.  NORTH
                    TETRAEDER(C,8,1) = CENTER(C)
                    TETRAEDER(C,8,2) = COORD(C,4)
                    TETRAEDER(C,8,3) = COORD(C,3)
                    TETRAEDER(C,8,4) = COORD(C,7)
C                   9.  SOUTH
                    TETRAEDER(C,9,1) = CENTER(C)
                    TETRAEDER(C,9,2) = COORD(C,1)
                    TETRAEDER(C,9,3) = COORD(C,5)
                    TETRAEDER(C,9,4) = COORD(C,6)
C                   10.  SOUTH
                    TETRAEDER(C,10,1) = CENTER(C)
                    TETRAEDER(C,10,2) = COORD(C,1)
                    TETRAEDER(C,10,3) = COORD(C,2)
                    TETRAEDER(C,10,4) = COORD(C,6)
C                   11.  BOTTOM
                    TETRAEDER(C,11,1) = CENTER(C)
                    TETRAEDER(C,11,2) = COORD(C,8)
                    TETRAEDER(C,11,3) = COORD(C,5)
                    TETRAEDER(C,11,4) = COORD(C,6)
C                   12.  BOTTOM
                    TETRAEDER(C,12,1) = CENTER(C)
                    TETRAEDER(C,12,2) = COORD(C,8)
                    TETRAEDER(C,12,3) = COORD(C,7)
                    TETRAEDER(C,12,4) = COORD(C,6)
            ENDDO          
          
      RETURN
      END
