      SUBROUTINE INSPH(A,B,C,D,CENTRE,RAD)

C WR      IMPLICIT LOGICAL (A-Z)
      IMPLICIT NONE
      
      DOUBLE PRECISION A(3),B(3),C(3),CENTRE(3),D(3),RAD
C
C     Written and copyright by:
C        Barry Joe, Dept. of Computing Science, Univ. of Alberta
C        Edmonton, Alberta, Canada  T6G 2H1
C        Phone: (403) 492-5757      Email: barry@cs.ualberta.ca
C
C     Purpose: Find centre and radius of insphere of tetrahedron.
C
C     Input parameters:
C        A(1:3),B(1:3),C(1:3),D(1:3) - 4 vertices of tetrahedron
C
C     Output parameters:
C        CENTRE(1:3) - centre of insphere; undefined if A,B,C,D coplanar
C        RAD - radius of insphere; 0 if A,B,C,D coplanar
C
      DOUBLE PRECISION PI,TOL
      PARAMETER(       PI=3.1415926535897932384626433832795D0,TOL=0.D0)
C WR      COMMON /GCONST/ PI,TOL
C WR      SAVE /GCONST/

C
      INTEGER I,J,R
      DOUBLE PRECISION AB(3),AC(3),AD(3),BC(3),BD(3),MAT(4,4),RTOL,T
      LOGICAL SINGLR
C
C     Compute unit outward (or inward) normals and equations of 4 faces.
C
      DO 10 I = 1,3
	 AB(I) = B(I) - A(I)
	 AC(I) = C(I) - A(I)
	 AD(I) = D(I) - A(I)
	 BC(I) = C(I) - B(I)
	 BD(I) = D(I) - B(I)
   10 CONTINUE
      RTOL = TOL*MAX(ABS(A(1)),ABS(A(2)),ABS(A(3)),ABS(B(1)),ABS(B(2)),
     $   ABS(B(3)),ABS(C(1)),ABS(C(2)),ABS(C(3)),ABS(D(1)),ABS(D(2)),
     $   ABS(D(3)))
      MAT(1,1) = AC(2)*AB(3) - AC(3)*AB(2)
      MAT(1,2) = AC(3)*AB(1) - AC(1)*AB(3)
      MAT(1,3) = AC(1)*AB(2) - AC(2)*AB(1)
      MAT(2,1) = AB(2)*AD(3) - AB(3)*AD(2)
      MAT(2,2) = AB(3)*AD(1) - AB(1)*AD(3)
      MAT(2,3) = AB(1)*AD(2) - AB(2)*AD(1)
      MAT(3,1) = AD(2)*AC(3) - AD(3)*AC(2)
      MAT(3,2) = AD(3)*AC(1) - AD(1)*AC(3)
      MAT(3,3) = AD(1)*AC(2) - AD(2)*AC(1)
      MAT(4,1) = BC(2)*BD(3) - BC(3)*BD(2)
      MAT(4,2) = BC(3)*BD(1) - BC(1)*BD(3)
      MAT(4,3) = BC(1)*BD(2) - BC(2)*BD(1)
      SINGLR = .TRUE.
      DO 30 I = 4,1,-1
	 T = SQRT(MAT(I,1)**2 + MAT(I,2)**2 + MAT(I,3)**2)
	 IF (T .LE. RTOL) GO TO 100
	 MAT(I,1) = MAT(I,1)/T
	 MAT(I,2) = MAT(I,2)/T
	 MAT(I,3) = MAT(I,3)/T
	 IF (I .EQ. 4) THEN
	    MAT(I,4) = MAT(I,1)*B(1) + MAT(I,2)*B(2) + MAT(I,3)*B(3)
	 ELSE
	    MAT(I,4) = MAT(I,1)*A(1) + MAT(I,2)*A(2) + MAT(I,3)*A(3)
	    DO 20 J = 1,4
	       MAT(I,J) = MAT(I,J) - MAT(4,J)
   20       CONTINUE
	 ENDIF
   30 CONTINUE
C
C     Use Gaussian elimination with partial pivoting to solve 3 by 3
C     system of linear equations for centre of insphere.
C
      R = 1
      DO 40 I = 2,3
	 IF (ABS(MAT(I,1)) .GT. ABS(MAT(R,1))) R = I
   40 CONTINUE
      IF (ABS(MAT(R,1)) .LE. TOL) GO TO 100
      IF (R .NE. 1) THEN
	 DO 50 J = 1,4
	    T = MAT(1,J)
	    MAT(1,J) = MAT(R,J)
	    MAT(R,J) = T
   50    CONTINUE
      ENDIF
      DO 70 I = 2,3
	 T = MAT(I,1)/MAT(1,1)
	 DO 60 J = 2,4
	    MAT(I,J) = MAT(I,J) - T*MAT(1,J)
   60    CONTINUE
   70 CONTINUE
      IF (ABS(MAT(3,2)) .GT. ABS(MAT(2,2))) THEN
	 DO 80 J = 2,4
	    T = MAT(2,J)
	    MAT(2,J) = MAT(3,J)
	    MAT(3,J) = T
   80    CONTINUE
      ENDIF
      IF (ABS(MAT(2,2)) .LE. TOL) GO TO 100
      T = MAT(3,2)/MAT(2,2)
      DO 90 J = 3,4
	 MAT(3,J) = MAT(3,J) - T*MAT(2,J)
   90 CONTINUE
      IF (ABS(MAT(3,3)) .GT. TOL) SINGLR = .FALSE.
C
  100 CONTINUE
      IF (SINGLR) THEN
	 RAD = 0.0D0
      ELSE
	 CENTRE(3) = MAT(3,4)/MAT(3,3)
	 CENTRE(2) = (MAT(2,4) - MAT(2,3)*CENTRE(3))/MAT(2,2)
	 CENTRE(1) = (MAT(1,4) - MAT(1,3)*CENTRE(3) -
     $      MAT(1,2)*CENTRE(2))/MAT(1,1)
	 RAD = ABS(MAT(4,1)*CENTRE(1) + MAT(4,2)*CENTRE(2) +
     $      MAT(4,3)*CENTRE(3) - MAT(4,4))
      ENDIF
      END
